﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Arction.LightningChartUltimate;
using Arction.LightningChartUltimate.SeriesXY;
using Arction.LightningChartUltimate.Axes;
using Arction.Licensing;
using ZMapReader;

namespace IntensityGridTest
{
    public partial class Form1 : Form
    {
        private LightningChartUltimate m_chart;
        private IntensityGridSeries m_intensityGrid = null;

        public Form1()
        {
            InitializeComponent();
        }
        public Form1(string[] args)
        {
            InitializeComponent();

            string path = "../../../data/autotest_anticlines_flowpath-Layer05-Depth-0Ma.dat";//low res
            //"../../../data/t10_v2010_autotest_anticlines_flowpath_0001.ele_Depth_layer_[3] Layer04 to Layer05@0.dat";//hi res
            if (args.Length > 0)
            {
                path = args[0];
            }

            try
            {
                ZmapReader zreader = new ZmapReader();
                zreader.path = path;
                zreader.ReadData();
                Populate("Test", zreader._x_ext, zreader._y_ext, zreader._dx, zreader._dy, zreader._x_min, zreader._y_min, zreader.Data);
            }
            catch (Exception e)
            {
                MessageBox.Show("Failed to show zmap. Reason: " + e.Message);
            }
            
        }
        private ValueRangePalette CreatePalette(Arction.LightningChartUltimate.SeriesXY.SeriesBaseXY ownerSeries, double ymin, double YRange)
        {
            ValueRangePalette palette = new ValueRangePalette(ownerSeries);
            palette.Type = PaletteType.Gradient;
            double yRange = YRange - ymin;
            palette.Steps.Clear();
            palette.MinValue = ymin;
            palette.Steps.Add(new PaletteStep(palette, Color.Blue, ymin));
            palette.Steps.Add(new PaletteStep(palette, Color.Lime, ymin + 33 * yRange / 100.0));
            palette.Steps.Add(new PaletteStep(palette, Color.Yellow, ymin + 66 * yRange / 100.0));
            palette.Steps.Add(new PaletteStep(palette, Color.Red, ymin + 100.0 * yRange / 100.0));


            return palette;
        }
        public void Populate(string title, int nX, int nY, double dx, double dy, double orgX, double orgY, float[] Z)
        {
            this.Text = title;
            m_chart = new LightningChartUltimate(LicenseKeys.LicenseKeyStrings.LightningChartUltimate);
            //Disable rendering, strongly recommended before updating chart properties
            m_chart.BeginUpdate();
            //Set active view
            m_chart.ActiveView = ActiveView.ViewXY;

            //Chart parent must be set
            m_chart.Parent = this;

            //Fill parent area with chart
            m_chart.Dock = DockStyle.Fill;

            //Setup legend box
            m_chart.ViewXY.LegendBox.Layout = LegendBoxLayout.Vertical;

            //Enable multi-core processing 
            m_chart.RenderOptions.MultiCoreProcessing = MultiCoreProcessing.AllAvailableCores;

            //Clear data from series
            m_chart.ViewXY.IntensityGridSeries.Clear();
            m_chart.Title.Text = title;
            //Create intensity grid and set some data 
            m_intensityGrid = new IntensityGridSeries(m_chart.ViewXY, m_chart.ViewXY.XAxes[0], m_chart.ViewXY.YAxes[0]);

            int iSizeX = nX;
            int iSizeY = nY;
            double normalRangeMaxX = orgX + ((nX - 1) * dx);
            double normalRangeMinX = orgX;
            double normalRangeMaxY = orgY + ((nY - 1) * dy);
            double normalRangeMinY = orgY;
            //Put some data

            double lX = normalRangeMaxX - normalRangeMinX;
            double lY = normalRangeMaxY - normalRangeMinY;
            double aspectratio = lX / lY;
            m_chart.ViewXY.ZoomPanOptions.AspectRatioOptions.AspectRatio = ViewAspectRatio.Manual;
            m_chart.ViewXY.ZoomPanOptions.AspectRatioOptions.ManualAspectRatioWH = aspectratio;

            float[] fz = Z.Where(z => !Double.IsNaN(z)).ToArray();
            double lZ = fz.Max() - fz.Min();

            //Setup x-axis
            m_chart.ViewXY.XAxes[0].ValueType = AxisValueType.Number;
            m_chart.ViewXY.XAxes[0].SetRange(orgX, orgX + ((nX - 1) * dx));
            
            //Setup y-axis
            m_chart.ViewXY.YAxes[0].SetRange(orgY, orgY + ((nY - 1) * dy));
            //m_chart.ViewXY.YAxes[0].Reversed = true;


            m_intensityGrid.SetRangesXY(orgX, orgX + ((nX - 1) * dx),orgY, orgY + ((nY - 1) * dy));
            m_intensityGrid.SetSize(iSizeX, iSizeY);
            //Add series to intensity mesh series list
            m_chart.ViewXY.IntensityGridSeries.Add(m_intensityGrid);

            IntensityPoint[,] data = new IntensityPoint[iSizeX, iSizeY];
            for (int iRow = 0; iRow < iSizeY; iRow++)
            {
                for (int iCol = 0; iCol < iSizeX; iCol++)
                {
                    data[iCol, (nY - 1) - iRow].Value = (Z[iCol * nY + iRow]);
                }
            }
            //Assign created data array to series
            m_intensityGrid.ShowNodes = false;
            m_intensityGrid.Data = data;
            m_intensityGrid.Fill = IntensityFillStyle.Paletted;
            m_intensityGrid.MouseInteraction = false;
            //m_intensityGrid.PixelRendering = true;
            m_intensityGrid.FullInterpolation = true;
            m_intensityGrid.Title.Text = title;
            m_intensityGrid.ValueRangePalette = CreatePalette(m_intensityGrid, fz.Min(), fz.Max());
            m_intensityGrid.InvalidateData();
            //Allow chart rendering
            m_chart.EndUpdate();
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            ToolStripButton item = (ToolStripButton)sender;
            item.Checked = !item.Checked;
            m_chart.BeginUpdate();
            m_intensityGrid.InvalidateData();
            m_intensityGrid.PixelRendering = item.Checked;
            //Allow chart rendering
            m_chart.EndUpdate();

        }
    }
}
