﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Arction.LightningChartUltimate;
using Arction.LightningChartUltimate.Series3D;
using Arction.LightningChartUltimate.Views.View3D;
using Arction.LightningChartUltimate.Axes;
using Arction.Licensing;
using System.Runtime.Serialization.Formatters.Binary;
using ZMapReader;

namespace SurfaceGrid3DTest
{
    public partial class Form1 : Form
    {
        private LightningChartUltimate m_chart = null;
        private float m_yscale = 25;

        public Form1()
        {
            InitializeComponent();
        }
        public Form1(string[] args)
        {
            InitializeComponent();

            string path = "../../../data/autotest_anticlines_flowpath-Layer05-Depth-0Ma.dat";//low res
            //"../../../data/t10_v2010_autotest_anticlines_flowpath_0001.ele_Depth_layer_[3] Layer04 to Layer05@0.dat";//hi res
            if (args.Length > 0)
            {
                path = args[0];
            }
            
            try
            {
                ZmapReader zreader = new ZmapReader();
                zreader.path = path;
                zreader.ReadData();                        
                Populate("Test", zreader._x_ext, zreader._y_ext,zreader._dx,zreader._dy,zreader._x_min,zreader._y_min,zreader.Data);
            }
            catch (Exception e)
            {
                MessageBox.Show("Failed to show zmap. Reason: " + e.Message);
            }
            
        }
        private ValueRangePalette CreatePalette(SeriesBase3D ownerSeries, double ymin, double YRange)
        {
            ValueRangePalette palette = new ValueRangePalette(ownerSeries);
            double yRange = YRange - ymin;
            palette.Steps.Clear();
            palette.MinValue = ymin;
            palette.Steps.Add(new PaletteStep(palette, Color.Blue, ymin));
            palette.Steps.Add(new PaletteStep(palette, Color.Lime, ymin + 40 * yRange / 100.0));
            palette.Steps.Add(new PaletteStep(palette, Color.Yellow, ymin + 60.0 * yRange / 100.0));
            palette.Steps.Add(new PaletteStep(palette, Color.Red, ymin + 100.0 * yRange / 100.0));
            palette.Type = PaletteType.Gradient;

            return palette;
        }

        public void Populate(string title, int nX, int nY, double dx, double dy, double orgX, double orgY, float[] Z)
        {

            //Create new chart 
            m_chart = new LightningChartUltimate(LicenseKeys.LicenseKeyStrings.LightningChartUltimate);

            //Disable rendering, strongly recommended before updating chart properties
            m_chart.BeginUpdate();

            //Set 3D as active view
            m_chart.ActiveView = ActiveView.View3D;

            //Chart parent must be set
            m_chart.Parent = this;

            //Fill parent area with chart
            m_chart.Dock = DockStyle.Fill;

            //Chart name
            m_chart.Name = title;
            m_chart.Title.Text = title;

            //Set x-rotation
            m_chart.View3D.Camera.RotationX = 38;
            m_chart.View3D.Camera.OrthographicCamera = true;
            //scale X and Y (X and Z on lightning chart) for correct aspect ratio
            double lX = ((nY - 1) * dy);
            double lZ = ((nX - 1) * dx);
            
            
            double maxdie = lZ;
            if (lX > lZ) maxdie = lX;
            m_chart.View3D.Dimensions.X = Convert.ToSingle(lX * 100.0 / maxdie);
            m_chart.View3D.Dimensions.Z = Convert.ToSingle(lZ * 100.0 / maxdie);
            
            m_chart.View3D.Dimensions.Y = m_yscale;
            
           
            //Set transparent walls 
            List<WallBase> walls = m_chart.View3D.GetWalls();
            foreach (WallBase wall in walls)
            {
                wall.Material.DiffuseColor = Color.FromArgb(30, wall.Material.DiffuseColor);
                wall.SetGridStripColor1(Color.FromArgb(60, wall.GetGridStripColor1()));
                wall.SetGridStripColor2(Color.FromArgb(60, wall.GetGridStripColor2()));
            }
            //Lightning Chart coords, our X = their Z, our Y = their X, our Z = their Y on 3D plots
            m_chart.View3D.XAxisPrimary3D.SetRange(orgY, orgY + ((nY - 1) * dy));
            m_chart.View3D.XAxisPrimary3D.Title.Text = "Y Axis";
            m_chart.View3D.XAxisPrimary3D.Reversed = true;
            m_chart.View3D.ZAxisPrimary3D.SetRange(orgX, orgX + ((nX - 1) * dx));
            m_chart.View3D.ZAxisPrimary3D.Title.Text = "X Axis";

            float[] fz = Z.Where(z => !Double.IsNaN(z)).ToArray();//filter out NaNs before getting Z (depth) min/max
            m_chart.View3D.YAxisPrimary3D.SetRange(fz.Min(), fz.Max());
            m_chart.View3D.YAxisPrimary3D.Reversed = true;//depths are positive values "Below Sea Level", so reverse
            m_chart.View3D.YAxisPrimary3D.Title.Text = "Burial Depth";

            //this doesn't seem to work as expected with reversed axis, seems to be lit from bottom
            m_chart.View3D.SetPredefinedLightingScheme(LightingScheme.DirectionalFromCamera);

            //Add 3D surface grid
            SurfaceGridSeries3D gridSeries = new SurfaceGridSeries3D(m_chart.View3D, Axis3DBinding.Primary, Axis3DBinding.Primary, Axis3DBinding.Primary);

            gridSeries.Title.Text = title;
            gridSeries.ContourPalette = CreatePalette(gridSeries, fz.Min(), fz.Max());
            gridSeries.ContourLineType = ContourLineType.ColorLine;
            gridSeries.ContourLineWidth = 2;
            gridSeries.WireframeType = SurfaceWireframeType.None;
            gridSeries.Fill = SurfaceFillStyle.Paletted;

            gridSeries.SetRangesXZ(orgY, orgY + ((nY - 1) * dy), orgX, orgX + ((nX - 1) * dx));
            gridSeries.SetSize(nY, nX);

            m_chart.View3D.SurfaceGridSeries3D.Add(gridSeries);
            //Lightning Chart coords, our X = their Z, our Y = their X, our Z = their Y on 3D plots            
            for (int iNodeX = 0; iNodeX < nY; iNodeX++)
            {
                for (int iNodeZ = 0; iNodeZ < nX; iNodeZ++)
                {
                    gridSeries.Data[(nY - 1) - iNodeX, iNodeZ].Y = (Z[iNodeZ * nY + iNodeX]);//stored column major
                }
            }
            //Notify new values are ready
            gridSeries.InvalidateData();

            //Allow chart rendering
            m_chart.EndUpdate();
        }
    }
}
